@@ -17,13 +17,16 @@ fn main() {
 
     Registry::new(Api::Gl, (3, 3), Profile::Core, Fallbacks::All, [
         "GL_ARB_blend_func_extended",
+        "GL_KHR_robustness",
         "GL_KHR_debug",
     ])
     .write_bindings(GlobalGenerator, &mut file)
     .unwrap();
 
     #[cfg(windows)]
-    embed_resource::compile("./windows/alacritty.rc", embed_resource::NONE);
+    embed_resource::compile("./windows/alacritty.rc", embed_resource::NONE)
+        .manifest_required()
+        .unwrap();
 }
 
 fn commit_hash() -> Option<String> {
