@@ -1,13 +1,14 @@
-use serde::{Deserialize, Deserializer};
+use serde::{Deserialize, Deserializer, Serialize};
 
 use alacritty_config_derive::{ConfigDeserialize, SerdeReplace};
 
 use crate::config::bindings::{self, MouseBinding};
 use crate::config::ui_config;
 
-#[derive(ConfigDeserialize, Default, Clone, Debug, PartialEq, Eq)]
+#[derive(ConfigDeserialize, Serialize, Default, Clone, Debug, PartialEq, Eq)]
 pub struct Mouse {
     pub hide_when_typing: bool,
+    #[serde(skip_serializing)]
     pub bindings: MouseBindings,
 }
 
