@@ -1,7 +1,9 @@
+use serde::Serialize;
+
 use alacritty_config_derive::ConfigDeserialize;
 use alacritty_terminal::term::SEMANTIC_ESCAPE_CHARS;
 
-#[derive(ConfigDeserialize, Clone, Debug, PartialEq, Eq)]
+#[derive(ConfigDeserialize, Serialize, Clone, Debug, PartialEq, Eq)]
 pub struct Selection {
     pub semantic_escape_chars: String,
     pub save_to_clipboard: bool,
