@@ -22,8 +22,8 @@ use std::fmt::{self, Display, Formatter};
 
 #[cfg(target_os = "macos")]
 use {
+    objc2::MainThreadMarker,
     objc2_app_kit::{NSColorSpace, NSView},
-    objc2_foundation::is_main_thread,
     winit::platform::macos::{OptionAsAlt, WindowAttributesExtMacOS, WindowExtMacOS},
 };
 
@@ -41,8 +41,8 @@ use winit::window::{
 use alacritty_terminal::index::Point;
 
 use crate::cli::WindowOptions;
-use crate::config::window::{Decorations, Identity, WindowConfig};
 use crate::config::UiConfig;
+use crate::config::window::{Decorations, Identity, WindowConfig};
 use crate::display::SizeInfo;
 
 /// Window icon for `_NET_WM_ICON` property.
@@ -198,7 +198,7 @@ impl Window {
         use_srgb_color_space(&window);
 
         let scale_factor = window.scale_factor();
-        log::info!("Window scale factor: {}", scale_factor);
+        log::info!("Window scale factor: {scale_factor}");
         let is_x11 = matches!(window.window_handle().unwrap().as_raw(), RawWindowHandle::Xlib(_));
 
         Ok(Self {
@@ -463,7 +463,7 @@ impl Window {
     pub fn set_has_shadow(&self, has_shadows: bool) {
         let view = match self.raw_window_handle() {
             RawWindowHandle::AppKit(handle) => {
-                assert!(is_main_thread());
+                assert!(MainThreadMarker::new().is_some());
                 unsafe { handle.ns_view.cast::<NSView>().as_ref() }
             },
             _ => return,
@@ -506,13 +506,11 @@ impl Window {
 fn use_srgb_color_space(window: &WinitWindow) {
     let view = match window.window_handle().unwrap().as_raw() {
         RawWindowHandle::AppKit(handle) => {
-            assert!(is_main_thread());
+            assert!(MainThreadMarker::new().is_some());
             unsafe { handle.ns_view.cast::<NSView>().as_ref() }
         },
         _ => return,
     };
 
-    unsafe {
-        view.window().unwrap().setColorSpace(Some(&NSColorSpace::sRGBColorSpace()));
-    }
+    view.window().unwrap().setColorSpace(Some(&NSColorSpace::sRGBColorSpace()));
 }
